// Ansteuerung eines HD44780 kompatiblen LCD im 4-Bit-Interfacemodus
// http://www.mikrocontroller.net/articles/AVR-GCC-Tutorial
//
// Die Pinbelegung ist ber defines in lcd-routines.h einstellbar
 
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "lcd-routines.h"
#include <util/delay.h>

// sendet ein Datenbyte an das LCD
void lcd_data(unsigned char temp1)
{
  LCD_PORT |= (1<<LCD_RS);        // RS auf 1 setzen --> = DATA byte will be send

  // Set all pins to low
  LCD_PORT &= ~( (1<<LCD_DB4) | (1<<LCD_DB5) | (1<<LCD_DB6) | (1<<LCD_DB7));
  if (temp1 & 0x80)
  {
    LCD_PORT |= (1<<LCD_DB7);
  }
  if (temp1 & 0x40)
  {
    LCD_PORT |= (1<<LCD_DB6);
  }
  if (temp1 & 0x20)
  {
    LCD_PORT |= (1<<LCD_DB5);
  }
  if (temp1 & 0x10)
  {
    LCD_PORT |= (1<<LCD_DB4);
  }
  lcd_enable();

  // Set all pins to low
  LCD_PORT &= ~( (1<<LCD_DB4) | (1<<LCD_DB5) | (1<<LCD_DB6) | (1<<LCD_DB7));
  if (temp1 & 0x08)
  {
    LCD_PORT |= (1<<LCD_DB7);
  }
  if (temp1 & 0x04)
  {
    LCD_PORT |= (1<<LCD_DB6);
  }
  if (temp1 & 0x02)
  {
    LCD_PORT |= (1<<LCD_DB5);
  }
  if (temp1 & 0x01)
  {
    LCD_PORT |= (1<<LCD_DB4);
  }
  lcd_enable();

  _delay_us(42);
}
 
// sendet einen Befehl an das LCD
void lcd_command(unsigned char temp1)
{
  LCD_PORT &= ~(1<<LCD_RS);        // RS auf 0 setzen --> = COMMAND byte will be send

  // Set all pins to low
  LCD_PORT &= ~( (1<<LCD_DB4) | (1<<LCD_DB5) | (1<<LCD_DB6) | (1<<LCD_DB7));
  if (temp1 & 0x80)
  {
    LCD_PORT |= (1<<LCD_DB7);
  }
  if (temp1 & 0x40)
  {
    LCD_PORT |= (1<<LCD_DB6);
  }
  if (temp1 & 0x20)
  {
    LCD_PORT |= (1<<LCD_DB5);
  }
  if (temp1 & 0x10)
  {
    LCD_PORT |= (1<<LCD_DB4);
  }
  lcd_enable();

  // Set all pins to low
  LCD_PORT &= ~( (1<<LCD_DB4) | (1<<LCD_DB5) | (1<<LCD_DB6) | (1<<LCD_DB7));
  if (temp1 & 0x08)
  {
    LCD_PORT |= (1<<LCD_DB7);
  }
  if (temp1 & 0x04)
  {
    LCD_PORT |= (1<<LCD_DB6);
  }
  if (temp1 & 0x02)
  {
    LCD_PORT |= (1<<LCD_DB5);
  }
  if (temp1 & 0x01)
  {
    LCD_PORT |= (1<<LCD_DB4);
  }
  lcd_enable();

  _delay_us(42);
}
 
// erzeugt den Enable-Puls
void lcd_enable(void)
{
  // Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers einfgen
  // http://www.mikrocontroller.net/topic/81974#685882
  LCD_PORT |= (1<<LCD_EN);
  // Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers verlngern
  // http://www.mikrocontroller.net/topic/80900
  _delay_us(4);                   // kurze Pause, evtl. reicht auch 1us
  LCD_PORT &= ~(1<<LCD_EN);
}
 
// Initialisierung: 
// Muss ganz am Anfang des Programms aufgerufen werden.
void lcd_init(void)
{
  LCD_DDR |= (1<<LCD_RS) | (1<<LCD_EN) | (1<<LCD_DB4) | (1<<LCD_DB5) | (1<<LCD_DB6) | (1<<LCD_DB7) | (1<<LCD_LIGHT);   // Port auf Ausgang schalten

  // Wait for more than 15ms after VDD rises to 4.5V
  _delay_ms(15);

  // Set initialisation data
  LCD_PORT &= ~( (1<<LCD_RS) | (1<<LCD_DB6) | (1<<LCD_DB7)); // set RS, DB6 and DB7 to "low"
  LCD_PORT |= (1<<LCD_DB4) | (1<<LCD_DB5);                   // set DB4 and DB5 to "high"

  // Notify display that data on bus is valid now
  lcd_enable();
 
  // Wait for more than 4.1ms
  _delay_ms(5);

  // Notify display that data on bus is valid now
  lcd_enable();
 
  // Wait for more than 100ms
  _delay_ms(1);

  // Notify display that data on bus is valid now
  lcd_enable();

  // just wait, because it seems to be usual
  _delay_ms(1);
 
  // activate 4bit mode (DB4, DB6 und DB7 = 0; DB5 = 1)
  LCD_PORT &= ~( (1<<LCD_DB4) | (1<<LCD_DB6) | (1<<LCD_DB7)); // set DB4, DB6 and DB7 to "low"
  LCD_PORT |= (1<<LCD_DB5);                                   // set DB5 to "high"
  lcd_enable();
  _delay_ms(1);
 
  // 4Bit / 2 lines / 5x7dots
  lcd_command(0x28); // --> see LCD204B_DIS spec page 5
    
  // Display on / Cursor off / Blinkinng off
  lcd_command(0x0C); // --> see LCD204B_DIS spec page 5
 
  // inkrement / kein Scrollen
  lcd_command(0x06); // --> see LCD204B_DIS spec page 5
 
  lcd_clear();
}
 
// Sendet den Befehl zur Lschung des Displays
void lcd_clear(void)
{
  lcd_command(CLEAR_DISPLAY);
  _delay_ms(5);
}
 
// Sendet den Befehl: Cursor Home
void lcd_home(void)
{
  lcd_command(CURSOR_HOME);
  _delay_ms(5);
}
 
// setzt den Cursor in Zeile y (1..4) Spalte x (0..15)
void set_cursor(uint8_t x, uint8_t y)
{
  uint8_t tmp;
 
  switch (y) 
  {
    case 1: tmp=0x80+0x00+x; break;    // 1. Zeile
    case 2: tmp=0x80+0x40+x; break;    // 2. Zeile
    case 3: tmp=0x80+0x14+x; break;    // 3. Zeile
    case 4: tmp=0x80+0x54+x; break;    // 4. Zeile
    default: return;                   // fr den Fall einer falschen Zeile
  }
  lcd_command(tmp);
}
 
// Schreibt einen String auf das LCD
void lcd_string(char *data)
{
  while(*data) 
  {
    lcd_data(*data);
    data++;
  }
}

void lcd_write_str(char *str)
{
	while (*str)
	{
		lcd_data(*str++);
	}
}

void lcd_write_P (const char *Buffer,...)
{
	va_list ap;
	va_start (ap, Buffer);	
	
	int format_flag;
	char str_buffer[10];
	char str_null_buffer[10];
	char move = 0;
	char Base = 0;
	int tmp = 0;
	char by;
	char *ptr;
		
	//Ausgabe der Zeichen
  for(;;)
	{
		by = pgm_read_byte(Buffer++);
		if(by==0) break; // end of format string
            
		if (by == '%')
		{
            by = pgm_read_byte(Buffer++);
			if (isdigit(by)>0)
				{
                                 
 				str_null_buffer[0] = by;
				str_null_buffer[1] = '\0';
				move = atoi(str_null_buffer);
                by = pgm_read_byte(Buffer++);
				}

			switch (by)
				{
        case 's':
          ptr = va_arg(ap,char *);
          while(*ptr) { lcd_data(*ptr++); }
          break;
      	case 'b':
					Base = 2;
					goto ConversionLoop;
				case 'c':
					//Int to char
					format_flag = va_arg(ap,int);
					lcd_data (format_flag++);
					break;
				case 'i':
					Base = 10;
					goto ConversionLoop;
				case 'o':
					Base = 8;
					goto ConversionLoop;
				case 'x':
					Base = 16;
					
          
        //****************************
        ConversionLoop:
        //****************************
        itoa(va_arg(ap,int),str_buffer,Base);
        int b=0;
        while (str_buffer[b++] != 0)
        {};

        b--;
        if (b<move)
        {
          move -=b;
          for (tmp = 0;tmp<move;tmp++)
          {
            str_null_buffer[tmp] = '0';
          }
          
          //tmp ++;
          str_null_buffer[tmp] = '\0';
          strcat(str_null_buffer,str_buffer);
          strcpy(str_buffer,str_null_buffer);
        }
        
        lcd_write_str (str_buffer);
        move =0;
        break;
      }
    }	
		else
		{
			lcd_data ( by );	
		}
	}
	va_end(ap);
}

void lcd_BackLightOn()
{
  LCD_PORT |= (1<<LCD_LIGHT);      // Backlight on
}

void lcd_BackLightOff()
{
  LCD_PORT &= ~(1<<LCD_LIGHT);      // Backlight off
}

//String aus EEPROM laden und an LCD senden
void lcd_eep_string(const unsigned char *data)
{	
  unsigned char c;
  while(1) 
  {
	  c = eeprom_read_byte(data);
		if(c==0) 
      return;
    lcd_data(c);
    data++;
  }
}

